

# Make the parent function readonly
--r() { readonly -f ${FUNCNAME[1]} 2> /dev/null && return 0; }


# Start function in background
--bg() { --r;
  { "$@"; } &
}


--close() { --r;

  if [[ ! $EXIT_FLAG -gt 0 ]]; # if first exit (background processes exiting)
    then
      if [[ $ERROR_FLAG -gt 0 ]];
        then
          --log err "Finished with $ERROR_FLAG errors."
        else
          --log success "Finished.";
      fi
      EXIT_FLAG=1; # to not print the msg again for every background process
  fi

  exit "$ERROR_FLAG";
}

# Show the version and license of Flying Tower
--version() { --r
  printf "%s v%s\n" "$APP_NAME" "$VERSION"
  printf "%s\n" "$LICENSE"
}

--help() { --r;

  --log help '
    verbose: [-v]
    version: [-V]
    debug  : [-d]
    help   : [-h]
    test   : [-t]

    key   : [-k <key>]
    lock  : [-l <path>]
    unlock: [-u <path>]
    output: [-o <path>]';
}


# Print messages if verbose is true or if it is an error and other cases.
# nocolor="\x1b[0m";
# red="\x1b[1;31m";
# green="\x1b[0;32m";
# yellow="\x1b[0;33m";
# blue="\x1b[1;34m";
--log() { --r;

  case ${1:-} in

    # without VERBOSE

    'err')
    declare -i -g ERROR_FLAG+=1;
    printf "\x1b[1;31m%-1s %-20s\x1b[0m\n" "[-]" "${2:-}" >&2;;

    'help')
    printf "\x1b[0;33m%s %s\x1b[0m\n" "[~]" "${2:-}" >&2;;

    'input')
    printf "\x1b[1;34m%s %s\x1b[0m\n" "[?]" "${2:-}" >&2;;

    # with VERBOSE

    'alert')
    [[ $VERBOSE_FLAG -gt 0 ]] && printf "\x1b[0;33m%s %s\x1b[0m\n" "[!]" "${2:-}" >&2;;

    'success')
    [[ $VERBOSE_FLAG -gt 0 ]] && printf "\x1b[0;32m%s %s\x1b[0m\n" "[+]" "${2:-}" >&2;;

    'jump')
    [[ $VERBOSE_FLAG -gt 0 ]] && printf "\n%s %s\n" "[.]" "${2:-}" >&2;;

    *)
    [[ $VERBOSE_FLAG -gt 0 ]] && printf "%s %s\n" "[.]" "${1:-}" >&2;;
  esac

return 0;
}


# Get password.
get_pass() { --r; local pass_='abcde' && local _word=123;

  # Return if pass already inputed
  if [[ ! $MASTER_KEY == '_...null..._' ]]; then
    return 0;
  else
    # Get and compare.
    until [[ "$pass_" == "$_word" ]]; do
      --log input 'Enter master password:'; read -r -s pass_;
      --log input 'Repeat master password:'; read -r -s _word;
    done;

    # Export pass to variable
    MASTER_KEY="$_word" && return 0;
  fi

  return 1;
}

# Delete files and folders (shred if possible)
--rm() { --r;

  # Check existence before
  if [[ ! -e "${1:-}" ]]; then return 0;
    else
      # If it is a file will try to shred or rm.
      if [[ -f "${1:-}" ]]; then
        shred -n 1 -z -u "${1:-}" &> /dev/null || rm -f "${1:-}" &> /dev/null;
      # Or case its a folder
      elif [[ -d "${1:-}" ]]; then
        rm -rf "${1:-}";
      fi
  fi

  # Check existence after
  if [[ -e "${1:-}" ]]; then
    --log err "Remove failed." && sleep 1 && --close;
  else
    return 0;
  fi
}
