
# Core loop
# Loops will call lock/unlock in the background for every path in the pool
# Will start second loop after all background processes of the other ends
package_loop() { --r; local x y;

    # Lock
    if [[ $LOCK_FLAG -gt 0 ]]; then
      --log "Will lock $LOCK_FLAG files/dirs"
      for x in "${LOCK_POOL[@]}"; do get_pass && --bg lock "$x"; done; wait;
    fi

    # Unlock
    if [[ $UNLOCK_FLAG -gt 0 ]]; then
      --log "Will unlock $UNLOCK_FLAG files/dirs"
      for y in "${UNLOCK_POOL[@]}"; do get_pass && --bg unlock "$y"; done; wait;
    fi
}

lock() { --r; local path working_dir file_name size output;

  # Functions
  # =========================================================================
  tar_and_enc() {
    tar -z -p -c "$file_name" 2> /dev/null | openssl enc -e -A -a -aes-256-cbc -k "$MASTER_KEY" > "${file_name}${SUFIX}";
  }

  # Get & Check path
  # =========================================================================
  path="$(get_absolut_path "${1:-}")";
  size="$(wc -c <"$path" 2> /dev/null)";
  working_dir="$(dirname "$path")";
  file_name="$(basename "$path")";
  output="./";

  check_absolut_path "$path" || return 1;

  # Starting
  # =========================================================================
  pushd "$working_dir" &> /dev/null;

  --log alert "Locking $path    Size: $((size / 1000000))M" || --log alert "Locking folder: $path";

  # Core
  # =========================================================================
  if tar_and_enc; then
    --log success "Locked ${file_name} to ${working_dir}/${file_name}${SUFIX}";
    --rm "$path";
    popd &> /dev/null;
    return 0;
  else
    --log err "Problem locking.";
    --rm "${path}${SUFIX}";
    popd &> /dev/null;
    return 1;
  fi
}

unlock() { --r; local path working_dir file_name size output;

  # Functions
  # =========================================================================
  dec_and_untar() {
    openssl enc -d -A -a -aes-256-cbc -k "$MASTER_KEY" < "$file_name" | tar -z -p -x -C "$output";
  }

  # Get & Check path
  # =========================================================================
  path="$(get_absolut_path "${1:-}")";
  size="$(wc -c <"$path" 2> /dev/null)";
  working_dir="$(dirname "$path")";
  file_name="$(basename "$path")";
  output="./";

  check_absolut_path "$path" --nofolder || return 1;

  # Starting
  # =========================================================================
  pushd "$working_dir" &> /dev/null;

  --log alert "Unlocking: $path    Size: $((size / 1000000))M";

  # Core
  # =========================================================================
  if dec_and_untar; then
    --log success "Unlocked ${file_name} to ${working_dir}/${file_name//.ft/}";
    --rm "$path";
    popd &> /dev/null;
    return 0;
  else
    --log err "Problem unlocking.";
    popd &> /dev/null;
    return 1;
  fi
}
