
set_globals() { --r;

  # Constants
  readonly APP_NAME="Flying Tower";
  readonly VERSION="0.1.0";
  readonly AUTHOR="Vitali Barozzi";
  readonly SUFIX='.ft';
  readonly LICENSE='
  Copyright (c) 2017 J. V. Vitali Barozzi

  Permission is hereby granted, free of charge, to any person obtaining a copy
  of this software and associated documentation files (the "Software"), to deal
  in the Software without restriction, including without limitation the rights
  to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
  copies of the Software, and to permit persons to whom the Software is
  furnished to do so, subject to the following conditions:

  The above copyright notice and this permission notice shall be included in all
  copies or substantial portions of the Software.

  THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
  IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
  FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
  AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
  LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
  OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
  SOFTWARE.'

  # Variables
  MASTER_KEY='_...null..._'
  OUTPUT=''

  # Flags start as integers equal to zero.
  declare -i -g {VERSION_FLAG,VERBOSE_FLAG,DEBUG_FLAG,HELP_FLAG,LOCK_FLAG,UNLOCK_FLAG,TEST_FLAG,ERROR_FLAG,EXIT_FLAG}=0;
}

set_flags() { --r; local arg flag;

  # if no arguments are passed will act as if -h was.
  [[ "$@" = '' ]] && HELP_FLAG=1;

  # Set nullglob up
  shopt -s nullglob

  while getopts "Vtvdhk:l:u:o:" flag; do
    case $flag in
      't') [[ $TEST_FLAG -eq 0 ]] && readonly TEST_FLAG=1 && readonly VERBOSE_FLAG=1;;
      'V') [[ $VERSION_FLAG -eq 0 ]] && readonly VERSION_FLAG=1;;
      'v') [[ $VERBOSE_FLAG -eq 0 ]] && readonly VERBOSE_FLAG=1;;
      'd') [[ $DEBUG_FLAG -eq 0 ]]   && readonly DEBUG_FLAG=1;;
      'k') readonly MASTER_KEY="${OPTARG}";;
      'o') readonly OUTPUT="${OPTARG}";;

      # Lock
      'l')
        for arg in $(eval echo "${OPTARG}"); do
          LOCK_POOL+=("${arg}") && LOCK_FLAG+=1;
        done;;

      # Unlock
      'u')
        for arg in $(eval echo "${OPTARG}"); do
          UNLOCK_POOL+=("${arg}") && UNLOCK_FLAG+=1;
        done;;

      'h'|*) [[ $HELP_FLAG -eq 0 ]] && readonly HELP_FLAG=1;;
    esac
  done

  # Set nullglob down
  shopt -u nullglob
}

prepare_system() { --r;

  # Soft strict mode
  set -uo pipefail && IFS=$'\n\t';

  # Set Debug
  [[ $DEBUG_FLAG -eq 1 ]] && set -x;

  # Set traps
  trap 'set +x; --close;' {0..27} || return 1;

  # HELP_FLAG early exit.
  [[ $HELP_FLAG -eq 1 ]] && { --help && --close; };

  # Version early exit.
  [[ $VERSION_FLAG -eq 1 ]] && { --version && --close; };

  # Check vital programs
  [[ -x $(command -v "openssl") ]] || { --log err "Need openssl" && --close; };
  [[ -x $(command -v "tar")     ]] || { --log err "Need tar"     && --close; };
  [[ -x $(command -v "dd")      ]] || { --log err "Need dd"      && --close; };

  # For testing
  [[ $TEST_FLAG -eq 1 ]] && { --test && --close; };

  --log jump "System ready." && return 0;
}
